import argparse
from github import Github, GithubException
import yaml
import base64
from datetime import datetime
import json

class DeployUpdate:
    def __init__(self, pat, metadata_file_path):
        self.pat = pat
        self.metadata_file_path = metadata_file_path
        self.g = Github(self.pat)

    @staticmethod
    def has_file_changes(old_content, new_content):
        return old_content.strip() != new_content.strip()

    def update_values_file(self, git_repo, file_path, image_key, new_version, commit_message, branch_name, source_branch):
        try:
            print(file_path, branch_name)
            contents = git_repo.get_contents(file_path, ref=branch_name)
            values_yaml = yaml.safe_load(base64.b64decode(contents.content))
            print(values_yaml)

            keys = image_key.split('.')
            temp = values_yaml
            for key in keys[:-1]:
                if key not in temp:
                    temp[key] = {}
                temp = temp[key]
            temp[keys[-1]] = new_version

            new_values_yaml = yaml.safe_dump(values_yaml, default_flow_style=False)
            print(new_values_yaml)

            if not self.has_file_changes(base64.b64decode(contents.content).decode('utf-8'), new_values_yaml):
                print("No changes detected, skipping update.")
                return None

            if source_branch == 'develop':
                git_repo.update_file(contents.path, commit_message, new_values_yaml, contents.sha, branch=branch_name)
                print(f"Directly updated {file_path} in {branch_name} branch.")
            else:
                target_branch = f"update-{image_key.replace('.', '-')}-{new_version}-{datetime.now().strftime('%Y%m%d%H%M%S')}"
                source_branch_ref = f"heads/{branch_name}"
                source_sha = git_repo.get_git_ref(source_branch_ref).object.sha
                git_repo.create_git_ref(ref=f"refs/heads/{target_branch}", sha=source_sha)
                git_repo.update_file(contents.path, commit_message, new_values_yaml, contents.sha, branch=target_branch)
                print(f"Updated {file_path} in new branch: {target_branch}.")
                return target_branch  
        except GithubException as e:
            print(f"Error updating values file: {e}")

    def create_pull_request(self, git_repo, title, body, head_branch, base_branch):
        try:
            pr = git_repo.create_pull(title=title, body=body, head=head_branch, base=base_branch)
            print(f"Pull request created: {pr.html_url}")
        except GithubException as e:
            print(f"Failed to create pull request: {e}")

    def get_config_for_branch(self, branch_name):
        try:
            with open(self.metadata_file_path, 'r') as file:
                metadata = json.load(file)
            if branch_name == 'develop':
                return metadata['environments']['dev']
            elif branch_name.startswith('release'):
                return metadata['environments']['staging']
            elif branch_name == 'main':
                return metadata['environments']['production']
            else:
                raise ValueError(f"Unknown branch name: {branch_name}")
        except Exception as e:
            print(f"Error loading metadata: {e}")
            return None

    def run(self, version, source_branch, application_name):
        user = self.g.get_user()
        print(f"Authenticated as: {user.login}")
        config = self.get_config_for_branch(source_branch)
        print(config)
        git_repo = self.g.get_repo(config['targetRepo'])

        updated_branch = self.update_values_file(git_repo, config['targetValuesPath'], config['imageKey'], version, f"[skip ci] Update the image version of {application_name} to {version}.", config['targetBranch'], source_branch)

        if updated_branch and source_branch != 'develop':
            self.create_pull_request(git_repo, f"[skip ci] Update the image version of {application_name} to {version}.", "[skip ci] Automated version update", updated_branch, config['targetBranch'])

def main():
    parser = argparse.ArgumentParser(description="Update Git repository values file and optionally create a pull request.")
    parser.add_argument('--pat', type=str, required=True, help='Personal Access Token for GitHub.')
    parser.add_argument('--version', type=str, required=True, help='New version to update in the values file.')
    parser.add_argument('--metadata_file_path', type=str, required=True, help='Path to the metadata configuration file.')
    parser.add_argument('--source_branch', type=str, required=True,  help='Source branch name.')
    parser.add_argument('--application_name', type=str, required=True, help="Application name")

    args = parser.parse_args()

    automation = DeployUpdate(args.pat, args.metadata_file_path)
    automation.run(args.version, args.source_branch, args.application_name)

if __name__ == '__main__':
    main()
